/*
 * smsalert: send a notification by SMS using Free mobile API
 *
 * Copyright (c) 2015 Arnaud Patard <arnaud.patard@rtp-net.org>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 *
 */
#define _GNU_SOURCE
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <libconfig.h>
#include <core.h>
#include <errno.h>

static int read_conf(char *fname, struct conf *opts)
{
	config_t cfg;
	int ret = 0;
	const char *str;
	int val;

	config_init(&cfg);

	ret = config_read_file(&cfg, fname);
	if (ret == CONFIG_FALSE) {
		switch (config_error_type(&cfg)) {
			default:
			case CONFIG_ERR_PARSE:
				fprintf(stderr, "%s:%d - %s\n", config_error_file(&cfg),
					config_error_line(&cfg), config_error_text(&cfg));
				ret = -EIO;
			case CONFIG_ERR_FILE_IO:
				ret = -ENOENT;
				break;
			case CONFIG_ERR_NONE:
				ret = 0;
				break;
		}
		goto exit_parse;
	}

	if (config_lookup_string(&cfg, "user", &str))
		opts->user = strdup(str);

	if (config_lookup_string(&cfg, "pass", &str))
		opts->pass = strdup(str);

	if (config_lookup_string(&cfg, "url", &str))
		opts->url = strdup(str);

	if (config_lookup_int(&cfg, "nosslerr", &val))
		opts->ignorecerterrors = val;

	ret = 0;

exit_parse:
	config_destroy(&cfg);
	return ret;
}

int main(int argc, char **argv) {
	struct conf opts = {
		.url = NULL,
		.ignorecerterrors = 0,
		.user = NULL,
		.pass = NULL,
		.verbose = 0,
	};
	int ret;
	char *fname;

	if (argc != 2) {
		fprintf(stderr, "smsalert <message>\n");
		exit(1);
	}

	ret = asprintf(&fname, "%s/.smsalert.cfg", secure_getenv("HOME"));
	if (ret < 0)
		exit(1);

	ret = read_conf(fname, &opts);
	if (ret < 0) {
		free(fname);
		fprintf(stderr, "Error while reading cfg %d\n", ret);
		exit(1);
	}
	free(fname);

	if (!opts.user || !opts.pass) {
		fprintf(stderr, "Missing configuration values\n");
		ret = 1;
		goto free_conf;
	}

	if (!opts.url)
		opts.url = strdup(DEFAULT_URL);

	ret = send_msg(&opts, argv[1]);
	if (ret < 0) {
		if (-ret > CURL_ERR_OFS)
			fprintf(stderr,"Curl returned %d\n", -ret-1024);
		else
			fprintf(stderr, "there was an error %d\n", ret);
		ret = 1;
		goto free_conf;

	}
	printf("Call returned %d\n", ret);
	ret = 0;

free_conf:
	if (opts.user)
		free(opts.user);
	if (opts.pass)
		free(opts.pass);
	if (opts.url)
		free(opts.url);
	return ret;
}
